%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% GyroPack V2
%%%
%%% Katie Poggensee, Bram Sterke, Heike Vallery
%%%
%%% This script loads and analyses the .mat files associated with the
%%% experiments on the balance beam.  
%%%
%%% Note: first the directory in the .path_data.txt has to be updated to
%%% your local directory of the .mat files.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
close all;
clc

%% Parameters 
sample_rate = 100;               %Hz

rad2deg = 360/(2*pi);
deg2rad = 2*pi/360;

sample_cutoff = 1 * sample_rate;         %1 second from end of recording
sample_length = 60 * sample_rate;        %60 seconds for the sample measurement

%setting filter params
Fc = 6;                             %Hz
Fs = sample_rate;                   %Hz

%optional params for filtcol_new function
% ord = 2                         %2nd order filter
% fb = []                         %default setting for forward-backward filtering

%conditions of tests
conditions = {'norm' 'unpo' 'low' 'high' 'neg' 'plac' 'unpo_end' 'norm_end'}; %'norm' 'unpo' 'low' 'high' 'neg' 'plac' 'unpo_end' 'norm_end'

%% Loading paths and data 

%adding path to VU3D model
addpath(genpath('C:\git\VU-3D-model\'));

%h1 = figure('units','normalized','outerposition',[0 0 1 1]);  %create figure for plotting to visual check
%vc_RHEE = input('visually inspect RHEE? (1/0) : ');
% vc_trunk = input('visually inspect trunk max/min/mean? (1/0) : ');

%% Load data
p_start = 1;
p_nr = 16;
trial_nr = 4;

for p_id = p_start:p_start+(p_nr-1)

    for condition_id = 1:length(conditions)

        for trial_id = 1:trial_nr
            %make filename
            filename = ['p' num2str(p_id) '_beam_' char(conditions(condition_id)) '_' num2str(trial_id)];

            %only run if filename exists, otherwise return
            fid = fopen('.path_data.txt','r');
            path = fscanf(fid,'%s');

            if isfile([path '\' filename '.mat'])

                data = load_data(filename);     %loading data
                condition_name = char(conditions(condition_id));
                fclose(fid); 
                
                %% Organize marker data
                mdata = data.(filename).Trajectories.Labeled.Data;                 %assign data
                mlabels = data.(filename).Trajectories.Labeled.Labels;             %assign labels
                mframes = data.(filename).Frames;                                  %length of measurement
                etimes = cell2mat({data.(filename).Events.Time});                  %assign event times
                elabels = convertCharsToStrings({data.(filename).Events.Label});   %assign event labels
                eframes = round(cell2mat({data.(filename).Events.Frame}),0);       %assign event frames

                %if no markers are present in qtm data an empty event is made to prevent errors in plotting
                if isempty(eframes)
                    etimes = zeros(1);
                    elabels = zeros(1);
                    eframes = zeros(1);
                end

                %remove the row of Residuals from the data
                mdata = permute(mdata, [1,3,2]);
                mdata(:,:,4) = [];
                mdata = permute(mdata, [1,3,2]);

                %% Assign data to variabels and cut sample to length

                for idx = 1:length(mlabels)
                    %filling temporary array and squeezing
                    temp = mdata(idx,1:3,:);
                    temp = squeeze(temp)';

                    %spline interpolate data and select sample
                    [temp_int,ones_gaps_limit_all,ones_gaps_all] = spline_interp_find_gaps(temp,25);     %interpolates (spline) all the gaps in the data (for each column)

                    if not(isnan(temp_int(20)))
                        [temp_filt] = filtcol_new(temp_int,Fs,Fc);                                          %filter columns

                        %populating markerlabels
                        eval([char(mlabels(idx)) '_filt = temp_filt;']);
                    end
                end

                %specify required labels for use in further functions
                required_labels = {'LASI_filt' 'RASI_filt' 'LPSI_filt' 'RPSI_filt' 'LSHO_filt' 'STRN_filt' 'C7_filt' 'RSHO_filt'};

                %create empty variables if labels don't exist
                for idx = 1:length(required_labels)
                    if ~exist(char(required_labels(idx)))
                        eval([char(required_labels(idx)) ' = [];']);
                    end
                end

                %visual inspection
                %subplot(2,2,1)
                
%                 if vc_RHEE
%                     subplot(2,1,1)
%                     plot(RHEE_filt(:,3))
%                     xline(eframes,'r')
%                     subplot(2,1,2)
%                     plot(LHEE_filt(:,3))
%                     xline(eframes,'r')
% 
%                     pause(1)
%                 end
            
                %calculate time between ML markers
                try
                    ML_time = etimes(2)-etimes(1);
                    if ML_time > 30
                        ML_time = 30;
                    end 
                catch
                    ML_time = 'NAN'
                end

                %calculate distance between last step and first step
                try
                    if elabels(4) == "br"
                        if strcmp(filename,'p11_beam_unpo_end_2')
                            R_distance = norm(RANK_filt(eframes(4)) - RANK_filt(eframes(3)));
                            L_distance = norm(RANK_filt(eframes(4)) - LANK_filt(eframes(3)));
                            beam_dist = min(5,min(R_distance,L_distance)/1000); %convert to meter and limit to 5m
                        else
                            R_distance = norm(RHEE_filt(eframes(4)) - RHEE_filt(eframes(3)));
                            L_distance = norm(RHEE_filt(eframes(4)) - LHEE_filt(eframes(3)));
                            beam_dist = min(5,min(R_distance,L_distance)/1000); %convert to meter and limit to 5m
                        end
                    elseif elabels(4) == "bl"
                        R_distance = norm(LHEE_filt(eframes(4)) - RHEE_filt(eframes(3)));
                        L_distance = norm(LHEE_filt(eframes(4)) - LHEE_filt(eframes(3)));
                        beam_dist = min(5,min(R_distance,L_distance)/1000); %convert to meter and limti to 5m
                    end
                catch
                    beam_dist = 'NAN'
                end

                %calculate time from AP marker to end
                try
                    AP_time = (mframes - eframes(5))/100;
                    if AP_time > 30
                        AP_time = 30;
                    end                  
                catch
                    AP_time = 'NAN'
                end

                %% creating trial array for exporting to R
                trial_headers = {'ID' 'condition' 'trial_nr' 'ML_time' 'beam_dist' 'AP_time'}; %'omega_stdmean'
                trial_array = {p_id char(conditions(condition_id)) trial_id ML_time beam_dist AP_time}; %trial_trunk(trial_id).stdmean

                if ~exist('output_mat')
                    output_mat = trial_array;
                else
                    output_mat = vertcat(output_mat, trial_array);
                end

                %p(p_id).(char(conditions(condition_id))).sw = trial_sw;   %assign SW trial struct to main structure
                %p(p_id).(char(conditions(condition_id))).sw_all_mean = mean([p(p_id).(char(conditions(condition_id))).sw.SW_all_mean]);

                %p(p_id).(char(conditions(condition_id))).omega = trial_trunk;   %assign trunk trial struct to main structure
                %p(p_id).(char(conditions(condition_id))).omega_std_all = mean([p(p_id).(char(conditions(condition_id))).omega.stdmean]);
            end

        end

    end

end

%% Exporting

T = array2table(output_mat);
T.Properties.VariableNames(1:length(trial_headers)) = trial_headers;
writetable(T,'O:\Healthy\Data_analyse\data_beam.csv')

fclose('all')

%% Preparing for doi publishing


