%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% GyroPack V2
%%%
%%% Katie Poggensee, Bram Sterke, Heike Vallery
%%%
%%% This script loads and analyses all .mat files associated with the
%%% experiments on the treadmill. 
%%%
%%% Note: first the directory in the .path_data.txt has to be updated to
%%% your local directory of the .mat files.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
close all;
clc

%% Parameters 
sample_rate = 100;               %Hz

rad2deg = 360/(2*pi);
deg2rad = 2*pi/360;

sample_cutoff = 0 * sample_rate;         %1 second from end of recording
sample_length = 60 * sample_rate;        %60 seconds for the sample measurement

%setting filter params
Fc = 6;                             %Hz
Fs = sample_rate;                   %Hz

%optional params for filtcol_new function
% ord = 2                         %2nd order filter
% fb = []                         %default setting for forward-backward filtering

%conditions of tests
conditions = {'norm' 'unpo' 'low' 'high' 'neg' 'plac' 'unpo_end' 'norm_end'}; %'norm' 'unpo' 'low' 'high' 'neg' 'plac' 'unpo_end' 'norm_end'
 
%counter for check of quality
quality_pelv = 0;
quality_neck = 0; 
quality_threshold = 1; %limit at 1% NaNs

%participant height
p_height = [];

%% Loading paths and data

%adding path to VU3D model
addpath(genpath('C:\git\VU-3D-model\'));

%% Load data
h1 = figure('units','normalized','outerposition',[0 0 1 1]);  %create figure for plotting to visual check

vc_hs = 0;      %input('visually inspect HS local minima detection? (1/0) : ');
vc_trunk = 0;   %input('visually inspect trunk max/min/mean? (1/0) : ');

p_start = 4;
p_nr = 13;
trial_nr = 4;

for p_id = p_start:p_start+(p_nr-1)

    for condition_id = 1:length(conditions)

        for trial_id = 1:trial_nr
            %make filename
            filename = ['p' num2str(p_id) '_tm_' char(conditions(condition_id)) '_' num2str(trial_id)];
           
            %only run if filename exists, otherwise return
            fid = fopen('.path_data.txt','r');
            path = fscanf(fid,'%s');

            if isfile([path '\' filename '.mat'])

                data = load_data(filename);     %loading data
                condition_name = char(conditions(condition_id));
                fclose(fid);  
                
                %% Organize marker data
                mdata = data.(filename).Trajectories.Labeled.Data;                 %assign data
                mlabels = data.(filename).Trajectories.Labeled.Labels;             %assign labels
                mframes = data.(filename).Frames;                                  %length of measurement
                etimes = cell2mat({data.(filename).Events.Time});                  %assign event times
                elabels = convertCharsToStrings({data.(filename).Events.Label});   %assign event labels
                eframes = round(cell2mat({data.(filename).Events.Frame}),0);       %assign event frames

                %if no markers are present in qtm data an empty event is made to prevent errors in plotting
                if isempty(eframes)
                    etimes = zeros(1);
                    elabels = zeros(1);
                    eframes = zeros(1);
                end

                %remove the row of Residuals from the data
                mdata = permute(mdata, [1,3,2]);
                mdata(:,:,4) = [];
                mdata = permute(mdata, [1,3,2]);
                
                %% Assign data to variabels and cut sample to length
                for idx = 1:length(mlabels)
                    %filling temporary array and squeezing
                    temp = mdata(idx,:,:);
                    temp = squeeze(temp)';
                    
                    %spline interpolate data and select sample
                    [temp_int,ones_gaps_limit_all,ones_gaps_all] = spline_interp_find_gaps(temp,25);     %interpolates (spline) all the gaps in the data (for each column)
                    
                     if ~isnan(temp_int(1))
                        [temp_filt] = filtcol_new(temp_int,Fs,Fc);                                          %filter columns
                     else 
                         temp_filt = temp_int;
                     end
                    
                    %reinsert the NaN's where gaps exceed limit
                    temp_filt(find(ones_gaps_limit_all == 1)) = NaN;
                    
                    %cutting last minute, 5 seconds from end
                    sample_start = (length(temp_filt)-(sample_length+sample_cutoff-1));
                    sample_end = (length(temp_filt)-sample_cutoff);
                    temp_filt = temp_filt(sample_start:sample_end,:);
                    
                    %populating markerlabels
                    eval([char(mlabels(idx)) '_filt = temp_filt;']);
                    
                end

                %cutting mdata to same sample length for later use
                mdata_cut = mdata(1:size(mdata,1),:,sample_start:sample_end);

                %specify required labels for use in further functions
                required_labels = {'LASI_filt' 'RASI_filt' 'LPSI_filt' 'RPSI_filt' 'LSHO_filt' 'STRN_filt' 'C7_filt' 'RSHO_filt'};

                %create empty variables if labels don't exist
                for idx = 1:length(required_labels)
                    if ~exist(char(required_labels(idx)))
                        eval([char(required_labels(idx)) ' = NaN(' num2str(sample_length) ',3);']);
                    end
                end

                %% get step width related data from RHEE and LHEE
                [trial_sw(trial_id), HS_L, HS_R, warn_max, warn_min] = calc_SW(RHEE_filt,LHEE_filt,sample_length,vc_hs);

                %produce warning if SW seems unreasonable
                if warn_max
                    warning(['Step width is above 250 in: ' filename])
                end
                if warn_min
                    warning(['Step width is below -60 in: ' filename])
                end

                %pausing when plotting for visual check of HS
                if vc_hs
                    pause(0.5)
                end

                %% creating center of pelvis and PELV points for use in trunk angular velocity calculations
                for idx = 1:sample_length
                    %calc center point of pelvis and neck (upper trunk)
                    %check which markers are available, if too many missing
                    %produce NaN. 
                    if (isnan(RASI_filt(idx,1)) || isnan(LPSI_filt(idx,1))) && (isnan(LASI_filt(idx,1)) || isnan(RPSI_filt(idx,1)))
                        PELV(idx,1:3) = NaN;
                        quality_pelv = quality_pelv + 1;
                    elseif isnan(RASI_filt(idx)) || isnan(LPSI_filt(idx))
                        PELV(idx,:) = (LASI_filt(idx,:) + RPSI_filt(idx,:))./2;
                    elseif isnan(LASI_filt(idx)) || isnan(RPSI_filt(idx))
                        PELV(idx,:) = (RASI_filt(idx,:) + LPSI_filt(idx,:))./2;
                    else
                        PELV(idx,:) = (LASI_filt(idx,:) + RASI_filt(idx,:) + LPSI_filt(idx,:) + RPSI_filt(idx,:))./4;
                    end
                    
                    %check which markers are available, if too many missing
                    %produce NaN. 
                    if (isnan(STRN_filt(idx,1)) || isnan(C7_filt(idx,1))) && (isnan(LSHO_filt(idx,1)) || isnan(RSHO_filt(idx,1)))
                        NECK(idx,1:3) = NaN;
                        quality_neck = quality_neck + 1;
                    elseif isnan(LSHO_filt(idx)) || isnan(RSHO_filt(idx))
                        NECK(idx,:) = (STRN_filt(idx,:) + C7_filt(idx,:))./2;
                    elseif isnan(STRN_filt(idx)) || isnan(C7_filt(idx))
                        NECK(idx,:) = (LSHO_filt(idx,:) + RSHO_filt(idx,:))./2;
                    else
                        NECK(idx,:) = (LSHO_filt(idx,:) + STRN_filt(idx,:) + C7_filt(idx,:) + RSHO_filt(idx,:))./4;
                    end
                end
                
                %spline interpolate data and select sample
                [NECK_int,ones_gaps_limit_all,ones_gaps_all] = spline_interp_find_gaps(NECK,25);     %interpolates (spline) all the gaps in the data (for each column)
                [NECK_filt] = filtcol_new(NECK_int,Fs,Fc);                                          %filter columns
                
                %reinsert the NaN's where gaps exceed limit
                NECK_filt(find(ones_gaps_limit_all == 1)) = NaN;
                        
                %spline interpolate data and select sample
                [PELV_int,ones_gaps_limit_all,ones_gaps_all] = spline_interp_find_gaps(PELV,25);     %interpolates (spline) all the gaps in the data (for each column)
                [PELV_filt] = filtcol_new(PELV_int,Fs,Fc);                                          %filter columns
                
                %reinsert the NaN's where gaps exceed limit
                PELV_filt(find(ones_gaps_limit_all == 1)) = NaN;
                        
                %% calculating trunk angular velocity
                
                %calculating trunk omega and derivatives
                [trial_trunk(trial_id), eX_trunk, eY_trunk, eZ_trunk, NECK_filt] = calc_trunkOmega(NECK_filt,PELV_filt,RSHO_filt,HS_L,Fs,vc_trunk);
                                
                %selecting maximum of all maxima
                trial_trunk_max(trial_id) = max([max(mean(trial_trunk(trial_id).data,1)),abs(min(mean(trial_trunk(trial_id).data,1)))]);
                trial_trunk_mean(trial_id) = mean(trial_trunk(trial_id).mean);
                
                %% quality check
                if ((quality_pelv*100/sample_length) > quality_threshold) || ((quality_neck*100/sample_length) > quality_threshold)
                    trial_trunk_max(trial_id) = NaN;
                    trial_trunk(trial_id).stdmean = NaN;
                    warning('warning: poor quality data')
                end
                
                %% creating trial array for exporting to R
                trial_headers = {'ID' 'condition' 'trial' 'sw_mean' 'sw_max' 'sw_min' 'sw_SD' 'sw_CV' 'omega_mean' 'omega_max' 'omega_SD'}; %'omega_stdmean'
                trial_array = {p_id char(conditions(condition_id)) trial_id trial_sw(trial_id).SW_all_mean  trial_sw(trial_id).SW_all_max trial_sw(trial_id).SW_all_min trial_sw(trial_id).SW_all_SD trial_sw(trial_id).SW_all_CV trial_trunk_mean(trial_id)  trial_trunk_max(trial_id) trial_trunk(trial_id).stdmean}; %trial_trunk(trial_id).stdmean

                if ~exist('output_mat')
                    output_mat = trial_array;
                else
                    output_mat = vertcat(output_mat, trial_array);
                end

                %p(p_id).(char(conditions(condition_id))).sw = trial_sw;   %assign SW trial struct to main structure
                %p(p_id).(char(conditions(condition_id))).sw_all_mean = mean([p(p_id).(char(conditions(condition_id))).sw.SW_all_mean]);

                %reset quality counters
                quality_pelv = 0;
                quality_neck = 0;
                
                %p(p_id).(char(conditions(condition_id))).omega = trial_trunk;   %assign trunk trial struct to main structure
                %p(p_id).(char(conditions(condition_id))).omega_std_all = mean([p(p_id).(char(conditions(condition_id))).omega.stdmean]);
            end

        end

    end

end

%% Exporting

T = array2table(output_mat);
T.Properties.VariableNames(1:length(trial_headers)) = trial_headers;
writetable(T,'O:\Healthy\Data_analyse\data_tm.csv')

fclose('all')
%% Plotting  %%%%%%%%%%%%%%%%%%%%
%% Plot all omega's per condition

%h2 = figure();

%C = fieldnames(p);
%
% hold on
% for p_id = 4:6
%     for idx = 1:numel(C)
%         F = C{idx};
%         plot(p_id,p(p_id).(F).omega_std_all,'*r')
%         text(p_id,p(p_id).(F).omega_std_all,F,'VerticalAlignment','bottom','HorizontalAlignment','right')
%     end
% end
% hold off

%% Plot RHEE and LHEE

% figure();
%
% hold on
% %     plot(RHEE_int(:,1))
% %     plot(RHEE_int(:,2))
% %     plot(RHEE_int(:,3))
%
%
%     plot(RHEE_filt(:,1))
%     plot(RHEE_filt(:,2))
%     plot(RHEE_filt(:,3))
%
%  %  legend('x-int','y-int','z-int','x-filt','y-filt','z-filt')
%
%     plot(LHEE_filt(:,1))
%     plot(LHEE_filt(:,2))
%     plot(LHEE_filt(:,3))
%
%     legend('rx','ry','rz','lx','ly','lz')
%     ylabel('distance (mm)')
%     xline(eframes,'r:',elabels) %vertical lineplot qtm events
%
% hold off
%
% if input('check trunk triad video? 1/0:')
%     %
%     h3 = figure()
%
%     for i = 1:sample_length/10
%
%         step = i*10;
%         %draw all markers
%         x = mdata_cut([1:size(mdata,1)],1,step);
%         y = mdata_cut([1:size(mdata,1)],2,step);
%         z = mdata_cut([1:size(mdata,1)],3,step);
%
%         plot3(x,y,z,'o')
%
%         hold on
%
%         plot3(PELV(step,1),PELV(step,2),PELV(step,3),'go')
%
%         %drawing triad for trunk
%         quiverInMatrix(PELV,eX_trunk,eY_trunk,eZ_trunk,step,100);
%
%         hold off
%
%         %figure settings
%         xlim([-500 1000])
%         ylim([-1000 1000])
%         zlim([-1000 2000])
%         grid on
%         xlabel('X')
%         ylabel('Y')
%         zlabel('Z')
%
%         drawnow;
%         pause(0.1)
%     end
% end

%% Plot trunk angular velocity (gOMEGAseg)

% gOMEGAseg_deg = rad2deg * gOMEGAseg;
% segOMEGAseg_deg = rad2deg * segOMEGAseg;
% x = 1:length(gOMEGAseg_sum);
%
% h3 = figure()
%
%     subplot(3,1,1)
%     plot(gOMEGAseg)
%     ylabel('angular velocity (deg/s)')
%     xlabel('time (ms)')
%     legend('x-global','y-global','z-global')
%
%     subplot(3,1,2)
%     plot(segOMEGAseg_deg)
%     ylabel('angular velocity (deg/s)')
%     xlabel('time (ms)')
%     legend('x-local','y-local','z-local')
%
%     subplot(3,1,3)
%     hold on
%         plot(gOMEGAseg_sum)
%         plot(x(HS_R),gOMEGAseg_sum(HS_R),'og')
%         plot(x(HS_L),gOMEGAseg_sum(HS_L),'or')
%     hold off
%     ylabel('angular velocity (deg/s)')
%     xlabel('time (ms)')
%     legend('x-local','HS_R','HS_L')

